/* Talika panel applet: Switch between open windows using icons.
 * (C) 2009-2010 Sinew Software Systems
	 *
 * Authors:  Vinod Kumar Dahiya
 *           Hemant Kumar
 *           Sigurdur Gudbrandsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
	 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <gtk/gtk.h>
#include <libwnck/screen.h>
#include <libwnck/libwnck.h>
#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <gconf/gconf-client.h>

#include "../config.h"
#include "tasklist.h"


#define PREF_DIALOG_UI_FILE   PACKAGE_DATA_DIR"/talika/talika_prefs.ui"
#define ICON_FILE   PACKAGE_DATA_DIR"/pixmaps/talika.svg" 


static void 
talika_prefrences_update_checkbuttons (TalikaTaskList * tasklist);


/*######################## Configuration with gconf ##########################*/

static void 
talika_show_all_workspaces_toggled (GtkToggleButton * button,
                                    PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "display_all_workspaces",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_group_windows_toggled (GtkToggleButton * button,
                              PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "group_windows",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_show_buttons_toggled (GtkToggleButton * button, 
                             PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "show_buttons",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_toggle_current_toggled (GtkToggleButton * button, 
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "highlight_current_window",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_dimm_minimised_toggled (GtkToggleButton * button,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "dimm_minimized_windows_icons",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_mono_icons_toggled (GtkToggleButton * button,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "mono_icons",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_enable_previews_toggled (GtkToggleButton * button,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "enable_previews",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_preview_rounded_corners_toggled (GtkToggleButton * button,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "preview_rounded_corners",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_preview_reverse_order_toggled (GtkToggleButton * button,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_bool (PANEL_APPLET (applet),
	                             "preview_reverse_order",
	                             gtk_toggle_button_get_active (button),
	                             NULL);
}

static void 
talika_preview_opacity_revalued (GtkAdjustment *adjustment,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_int (PANEL_APPLET (applet),
	                             "preview_opacity",
	                             gtk_adjustment_get_value (adjustment),
	                             NULL);
}

static void 
talika_preview_rows_revalued (GtkAdjustment *adjustment,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_int (PANEL_APPLET (applet),
	                             "preview_rows",
	                             gtk_adjustment_get_value (adjustment),
	                             NULL);
}

static void 
talika_preview_cols_revalued (GtkAdjustment *adjustment,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_int (PANEL_APPLET (applet),
	                             "preview_cols",
	                             gtk_adjustment_get_value (adjustment),
	                             NULL);
}

static void 
talika_preview_size_revalued (GtkAdjustment *adjustment,
                               PanelApplet * applet)
{
	panel_applet_gconf_set_int (PANEL_APPLET (applet),
	                             "preview_size",
	                             gtk_adjustment_get_value (adjustment),
	                             NULL);
}

static void 
talika_launchers_list_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry,
                               TalikaTaskList * tasklist)
{
	GSList *value;
	
	if (!entry->value || entry->value->type != GCONF_VALUE_LIST)
		return;

	//value = gconf_value_get_list (entry->value);

	value =
		panel_applet_gconf_get_list(tasklist->applet,
		                             "launchers",
		                             GCONF_VALUE_STRING,
		                             NULL);

	talika_tasklist_update_launchers_list (tasklist, value);
}

static void 
talika_display_all_workspaces_changed (GConfClient * client,
                                       guint cnxn_id,
                                       GConfEntry * entry,
                                       TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->include_all_workspaces = (value != 0);
	talika_tasklist_update_show_all_workspaces (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_group_windows_changed (GConfClient * client,
                              guint cnxn_id,
                              GConfEntry * entry,
                              TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->use_grouping = (value != 0);
	talika_tasklist_update_group_windows (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_button_relief_changed (GConfClient * client,
                              guint cnxn_id,
                              GConfEntry * entry, 
                              TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->button_relief = (value != 0);
	talika_tasklist_update_button_relief (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_dimm_minimised_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->dimm_minimised = (value != 0);
	talika_tasklist_update_dimm_minimised (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_toggle_current_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->toggle_current_task = (value != 0);
	talika_tasklist_update_toggle_current_task (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_mono_icons_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->mono_icons = (value != 0);
	talika_tasklist_update_mono_icons (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_enable_previews_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->enable_previews = (value != 0);
	talika_tasklist_update_enable_previews (tasklist);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_rounded_corners_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->preview_rounded_corners = (value != 0);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_reverse_order_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gboolean value;

	if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
		return;

	value = gconf_value_get_bool (entry->value);

	tasklist->preview_reverse_order = (value != 0);

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_opacity_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gint value;

	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
		return;

	value = gconf_value_get_int (entry->value);

	tasklist->preview_transparency = value;

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_rows_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gint value;

	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
		return;

	value = gconf_value_get_int (entry->value);

	tasklist->preview_rows = value;

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_cols_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gint value;

	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
		return;

	value = gconf_value_get_int (entry->value);

	tasklist->preview_cols = value;

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_preview_size_changed (GConfClient * client,
                               guint cnxn_id,
                               GConfEntry * entry, 
                               TalikaTaskList * tasklist)
{
	gint value;

	if (!entry->value || entry->value->type != GCONF_VALUE_INT)
		return;

	value = gconf_value_get_int (entry->value);

	tasklist->preview_size = value;

	talika_prefrences_update_checkbuttons (tasklist);
}

static void 
talika_setup_gconf (PanelApplet * applet, 
                    TalikaTaskList * tasklist)
{
	GConfClient *client;
	char *key;

	client = gconf_client_get_default ();

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "display_all_workspaces");
	tasklist->gconf_listeners[0] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_display_all_workspaces_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "dimm_minimized_windows_icons");
	tasklist->gconf_listeners[1] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_dimm_minimised_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet), "show_buttons");
	tasklist->gconf_listeners[2] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_button_relief_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "highlight_current_window");
	tasklist->gconf_listeners[3] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_toggle_current_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "group_windows");
	tasklist->gconf_listeners[4] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_group_windows_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	
	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "mono_icons");
	tasklist->gconf_listeners[5] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_mono_icons_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "enable_previews");
	tasklist->gconf_listeners[6] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_enable_previews_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_rounded_corners");
	tasklist->gconf_listeners[7] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_rounded_corners_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_reverse_order");
	tasklist->gconf_listeners[8] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_reverse_order_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_opacity");
	tasklist->gconf_listeners[9] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_opacity_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_rows");
	tasklist->gconf_listeners[10] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_rows_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_cols");
	tasklist->gconf_listeners[11] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_cols_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "preview_size");
	tasklist->gconf_listeners[12] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_preview_size_changed,
		                         tasklist, NULL, NULL);
	g_free (key);

	key = panel_applet_gconf_get_full_key (PANEL_APPLET
	                                       (applet),
	                                       "launchers");
	tasklist->gconf_listeners[13] =
		gconf_client_notify_add (client, key,
		                         (GConfClientNotifyFunc)
		                         talika_launchers_list_changed,
		                         tasklist, NULL, NULL);
	g_free (key);
	
	g_object_unref (G_OBJECT (client));
}

/*####################### Prefrences/Properties Dialog #######################*/

static void 
talika_prefrences_update_checkbuttons (TalikaTaskList * tasklist)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->include_all_workspaces_button),
	                              tasklist->include_all_workspaces);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->group_windows_button),
	                              tasklist->use_grouping);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->dimm_minimised_button),
	                              tasklist->dimm_minimised);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->toggle_current_task_button),
	                              tasklist->toggle_current_task);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->button_relief_button),
	                              tasklist->button_relief);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->mono_icons_button),
	                              tasklist->mono_icons);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->enable_preview_button),
	                              tasklist->enable_previews);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->preview_rounded_corners_button),
	                              tasklist->preview_rounded_corners);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
	                              (tasklist->preview_reverse_order_button),
	                              tasklist->preview_reverse_order);

	gtk_adjustment_set_value (tasklist->preview_opacity_adj,
	                          tasklist->preview_transparency);
	gtk_adjustment_set_value (tasklist->preview_rows_adj,
	                          tasklist->preview_rows);
	gtk_adjustment_set_value (tasklist->preview_cols_adj,
	                          tasklist->preview_cols);
	gtk_adjustment_set_value (tasklist->preview_size_adj,
	                          tasklist->preview_size);

	gtk_widget_set_sensitive (tasklist->preview_frame,
	                          tasklist->enable_previews);
}

static void 
talika_setup_properties_dialog (PanelApplet * applet,
                                TalikaTaskList * tasklist)
{
	if (tasklist->properties_dialog == NULL)
	{
		GtkBuilder *builder;
		GError *error;

		builder = gtk_builder_new ();
		gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);

		error = NULL;
		gtk_builder_add_from_file (builder,
		                           PREF_DIALOG_UI_FILE,
		                           &error);

		if (error)
		{
			g_warning ("Error loading preferences: %s",
			           error->message);
			g_error_free (error);
			return;
		}

		tasklist->properties_dialog =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "talika_prefs_dialogs");

		g_object_add_weak_pointer (G_OBJECT
		                           (tasklist->properties_dialog),
		                           (void **)
		                           &tasklist->properties_dialog);

		gtk_window_set_icon_from_file (GTK_WINDOW
		                               (tasklist->properties_dialog),
		                               ICON_FILE,
		                               NULL);

		g_signal_connect (G_OBJECT (tasklist->properties_dialog),
		                  "delete-event",
		                  G_CALLBACK (gtk_widget_hide), NULL);

		g_signal_connect_swapped (G_OBJECT
		                          (gtk_builder_get_object
		                           (builder, "close_button")),
		                          "clicked",
		                          G_CALLBACK (gtk_widget_hide),
		                          tasklist->properties_dialog);

		//get check all checkbuttons
		tasklist->include_all_workspaces_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "include_all_workspaces");
		tasklist->group_windows_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "group_windows");
		tasklist->dimm_minimised_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "dimm_minimised");
		tasklist->button_relief_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "show_buttons");
		tasklist->toggle_current_task_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "toggle_current");

		tasklist->mono_icons_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "mono_icons");
		tasklist->enable_preview_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "enable_previews");
		tasklist->preview_frame =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "preview_frame");
		tasklist->preview_rounded_corners_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "preview_rounded_corners");
		tasklist->preview_reverse_order_button =
			(GtkWidget *) gtk_builder_get_object (builder,
			                                      "preview_reverse_order");

		tasklist->preview_opacity_adj =
			(GtkAdjustment *) gtk_builder_get_object (builder,
			                                      "preview_opacity");
		tasklist->preview_rows_adj =
			(GtkAdjustment *) gtk_builder_get_object (builder,
			                                      "preview_rows");
		tasklist->preview_cols_adj =
			(GtkAdjustment *) gtk_builder_get_object (builder,
			                                      "preview_cols");
		tasklist->preview_size_adj =
			(GtkAdjustment *) gtk_builder_get_object (builder,
			                                      "preview_size");

		//update button states
		talika_prefrences_update_checkbuttons (tasklist);

		//connect signal handlers
		g_signal_connect (G_OBJECT
		                  (tasklist->include_all_workspaces_button),
		                  "toggled",
		                  G_CALLBACK
		                  (talika_show_all_workspaces_toggled),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->group_windows_button),
		                  "toggled",
		                  G_CALLBACK
		                  (talika_group_windows_toggled),
		                  applet);
		g_signal_connect (G_OBJECT (tasklist->dimm_minimised_button),
		                  "toggled",
		                  G_CALLBACK (talika_dimm_minimised_toggled),
		                  applet);
		g_signal_connect (G_OBJECT (tasklist->button_relief_button),
		                  "toggled",
		                  G_CALLBACK (talika_show_buttons_toggled),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->toggle_current_task_button),
		                  "toggled",
		                  G_CALLBACK (talika_toggle_current_toggled),
		                  applet);

		g_signal_connect (G_OBJECT
		                  (tasklist->mono_icons_button),
		                  "toggled",
		                  G_CALLBACK (talika_mono_icons_toggled),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->enable_preview_button),
		                  "toggled",
		                  G_CALLBACK (talika_enable_previews_toggled),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->preview_rounded_corners_button),
		                  "toggled",
		                  G_CALLBACK (talika_preview_rounded_corners_toggled),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->preview_reverse_order_button),
		                  "toggled",
		                  G_CALLBACK (talika_preview_reverse_order_toggled),
		                  applet);

		g_signal_connect (G_OBJECT
		                  (tasklist->preview_opacity_adj),
		                  "value-changed",
		                  G_CALLBACK (talika_preview_opacity_revalued),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->preview_rows_adj),
		                  "value-changed",
		                  G_CALLBACK (talika_preview_rows_revalued),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->preview_cols_adj),
		                  "value-changed",
		                  G_CALLBACK (talika_preview_cols_revalued),
		                  applet);
		g_signal_connect (G_OBJECT
		                  (tasklist->preview_size_adj),
		                  "value-changed",
		                  G_CALLBACK (talika_preview_size_revalued),
		                  applet);

		g_object_unref (builder);
	}
}

/*######################## Context Menu ######################################*/

static void 
display_properties_dialog (BonoboUIComponent * uic,
                           gpointer data, 
                           const gchar * verbname)
{
	TalikaTaskList *tasklist = data;
	gtk_window_present (GTK_WINDOW (tasklist->properties_dialog));
}

static void 
display_about_dialog ()
{
	static const gchar *authors[] = {
		"Lead Developers:",
		"Vinod Kumar Dahiya <vinod@sinewmail.com>",
		"Hemant Kumar <hemant@sinewmail.com>",
		"",
		"Contributers:",
		"Sigurdur Gudbrandsson <nikkiclau@gmail.com>",
		NULL
	};

	/*const gchar *documenters[] = {
		* "Vinod Kumar Dahiya <vinod@sinewmail.com>",
	 * "Hemant Kumar <hemant@sinewmail.com>",
	 * NULL
	 * }; */

	const gchar* translaters = "Vinod Kumar Dahiya <vinod@sinewmail.com> (Hindi)\n"
		"Mario Blättermann <mariobl@gnome.org> (Germen)\n"
		"Angelo Campanelli <acidburn_work@hotmail.com> (Italian)\n"
		"Arve Eriksson <031299870@telia.com> (Swedish)\n"
		"Tomasz Chrzczonowicz <tomasz@chrzczonowicz.com> (Polish)\n"
		"Alexander Tjaer <sashptz@gmail.com> (Russian)\n"
		"Roberto Rodríguez Pino <rodpin@gmail.com> (Spanish)\n"
		"Louis Scheidt <louis.scheidt@free.fr> (French)\n"
		"Atilla ÖNTAŞ <atilla_ontas@mandriva.org> (Turkish)\n";

	GtkWidget *about_dialog;

	about_dialog = gtk_about_dialog_new ();

	gtk_window_set_icon_from_file (GTK_WINDOW (about_dialog),
	                               ICON_FILE, NULL);

	g_object_set (about_dialog,
	              "program-name", _("Talika"),
	              "comments",
	              _("Switch between open windows using icons"),
	              "version", PACKAGE_VERSION, "copyright",
	              "\xC2\xA9 2009-2010 Sinew Software Systems",
	              "website",PACKAGE_BUGREPORT,
	              "authors", authors,"translator-credits", translaters,
	              "logo-icon-name", "talika", NULL);

	gtk_widget_show (about_dialog);

	g_signal_connect (about_dialog, "response",
	                  G_CALLBACK (gtk_widget_destroy), NULL);


	gtk_window_present (GTK_WINDOW (about_dialog));
}

const char *context_menu_translation_dummy[] =
{
	N_("_Preferences"),
	N_("_About")
};

static const char context_menu_xml[] =
"<popup name=\"button3\">\n"
"   <menuitem name=\"Properties Item\" "
"             verb=\"TalikaProperties\" "
"           _label=\"_Preferences\"\n"
"          pixtype=\"stock\" "
"          pixname=\"gtk-properties\"/>\n"
"   <menuitem name=\"About Item\" "
"             verb=\"TalikaAbout\" "
"           _label=\"_About\"\n"
"          pixtype=\"stock\" "
"          pixname=\"gtk-about\"/>\n" "</popup>\n";


static const BonoboUIVerb talika_menu_verbs[] = {
	BONOBO_UI_VERB ("TalikaProperties", display_properties_dialog),
	BONOBO_UI_VERB ("TalikaAbout", display_about_dialog),
	BONOBO_UI_VERB_END
};


/*########################## Applet Callbacks ################################*/

static void 
talika_orientation_changed (PanelApplet * applet, 
                            PanelAppletOrient orient,
                            gpointer data)
{
	TalikaTaskList *tasklist = data;

	if( talika_tasklist_change_orient(tasklist,orient) )
	{
		gtk_container_add (GTK_CONTAINER (applet), tasklist->box);
		gtk_widget_show_all (GTK_WIDGET (applet));
	}
}

static void 
talika_size_request (GtkWidget * applet,
                     GtkRequisition * requisition, 
                     TalikaTaskList * tasklist)
{
	int size_hints[2];

	talika_tasklist_size_allocation (tasklist, size_hints);

	panel_applet_set_size_hints (PANEL_APPLET (applet), size_hints, 2, 0);
}

/*############################## Applet ######################################*/

static void 
talika_destroy (GtkWidget * list, gpointer data)
{
	TalikaTaskList *tasklist = data;

	GConfClient *client = gconf_client_get_default ();

	gconf_client_notify_remove (client, tasklist->gconf_listeners[0]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[1]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[2]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[3]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[4]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[5]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[6]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[7]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[8]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[9]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[10]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[11]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[12]);
	gconf_client_notify_remove (client, tasklist->gconf_listeners[13]);

	g_object_unref (G_OBJECT (client));

	tasklist->gconf_listeners[0] = 0;
	tasklist->gconf_listeners[1] = 0;
	tasklist->gconf_listeners[2] = 0;
	tasklist->gconf_listeners[3] = 0;
	tasklist->gconf_listeners[4] = 0;
	tasklist->gconf_listeners[5] = 0;
	tasklist->gconf_listeners[6] = 0;
	tasklist->gconf_listeners[7] = 0;
	tasklist->gconf_listeners[8] = 0;
	tasklist->gconf_listeners[9] = 0;
	tasklist->gconf_listeners[10] = 0;
	tasklist->gconf_listeners[11] = 0;
	tasklist->gconf_listeners[12] = 0;
	tasklist->gconf_listeners[13] = 0;

	if (tasklist->properties_dialog)
		gtk_widget_destroy (tasklist->properties_dialog);

	talika_tasklist_free (tasklist);
}

static gboolean 
talika_applet_fill (PanelApplet * applet,
                    const gchar * iid, 
                    gpointer data)
{
	TalikaTaskList *tasklist;

	if (strcmp (iid, "OAFIID:TalikaApplet") != 0)
		return FALSE;

	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);

	panel_applet_set_flags (applet, PANEL_APPLET_HAS_HANDLE
	                        | PANEL_APPLET_EXPAND_MAJOR
	                        | PANEL_APPLET_EXPAND_MINOR);

	//add gconf prefrences
	panel_applet_add_preferences (applet,
	                              "/schemas/apps/talika/prefs", NULL);

	//create tasklist
	WnckScreen *screen;
	GdkScreen *gdkscreen;
	gdkscreen = gtk_widget_get_screen (GTK_WIDGET (applet));
	screen = wnck_screen_get (gdk_screen_get_number (gdkscreen));

	tasklist = talika_tasklist_new (screen,
	                                panel_applet_get_size (applet),
	                                panel_applet_get_orient (applet));

	tasklist->applet = applet;

	//setup gconf-listeners and get keys 
	talika_setup_gconf (applet, tasklist);

	GError *error;

	error = NULL;
	tasklist->include_all_workspaces =
		panel_applet_gconf_get_bool (applet, "display_all_workspaces",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->include_all_workspaces = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->use_grouping =
		panel_applet_gconf_get_bool (applet, "group_windows",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->use_grouping = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->button_relief =
		panel_applet_gconf_get_bool (applet, "show_buttons", &error);
	if (error)
	{
		g_error_free (error);
		tasklist->button_relief = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->toggle_current_task =
		panel_applet_gconf_get_bool (applet,
		                             "highlight_current_window",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->toggle_current_task = TRUE;	/* Default value */
	}

	error = NULL;
	tasklist->dimm_minimised =
		panel_applet_gconf_get_bool (applet,
		                             "dimm_minimized_windows_icons",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->dimm_minimised = TRUE;	/* Default value */
	}

	error = NULL;
	tasklist->mono_icons =
		panel_applet_gconf_get_bool (applet,
		                             "mono_icons",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->mono_icons = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->enable_previews =
		panel_applet_gconf_get_bool (applet,
		                             "enable_previews",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->enable_previews = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->preview_rounded_corners =
		panel_applet_gconf_get_bool (applet,
		                             "preview_rounded_corners",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_rounded_corners = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->preview_reverse_order =
		panel_applet_gconf_get_bool (applet,
		                             "preview_reverse_order",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_reverse_order = FALSE;	/* Default value */
	}

	error = NULL;
	tasklist->preview_transparency =
		panel_applet_gconf_get_int (applet,
		                             "preview_opacity",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_transparency = 9;	/* Default value */
	}

	error = NULL;
	tasklist->preview_rows =
		panel_applet_gconf_get_int (applet,
		                             "preview_rows",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_rows = 2;	/* Default value */
	}

	error = NULL;
	tasklist->preview_cols =
		panel_applet_gconf_get_int (applet,
		                             "preview_cols",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_cols = 4;	/* Default value */
	}

	error = NULL;
	tasklist->preview_size =
		panel_applet_gconf_get_int (applet,
		                             "preview_size",
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->preview_size = 120;	/* Default value */
	}

	error = NULL;
	tasklist->launchers_ids =
		panel_applet_gconf_get_list(applet,
		                             "launchers",
		                             GCONF_VALUE_STRING,
		                             &error);
	if (error)
	{
		g_error_free (error);
		tasklist->launchers_ids = NULL;	/* Default value */
	}
	
	//update tasklist now to reflect new prefrence values
	talika_tasklist_update_show_all_workspaces (tasklist);

	talika_setup_properties_dialog (applet, tasklist);

	//FIXME: Why it is here (i forget, may be to track panel size) ?
	talika_orientation_changed (applet, panel_applet_get_orient (applet),
	                            tasklist);

	//remove preview cache if not enabled
	talika_tasklist_update_enable_previews(tasklist);

	//do the rest
	gtk_container_add (GTK_CONTAINER (applet), tasklist->box);

	gtk_widget_show_all (GTK_WIDGET (applet));

	g_signal_connect (PANEL_APPLET (applet), "change-orient",
	                  G_CALLBACK (talika_orientation_changed), tasklist);
	g_signal_connect (PANEL_APPLET (applet), "destroy",
	                  G_CALLBACK (talika_destroy), tasklist);
	g_signal_connect (G_OBJECT (applet), "size-request",
	                  G_CALLBACK (talika_size_request), tasklist);

	panel_applet_set_background_widget (PANEL_APPLET (applet),
	                                    GTK_WIDGET (applet));

	panel_applet_setup_menu (PANEL_APPLET (applet),
	                         context_menu_xml, talika_menu_verbs,
	                         tasklist);

	return TRUE;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:TalikaApplet_Factory",
                             PANEL_TYPE_APPLET,
                             "Switch between open windows using icons",
                             "0", talika_applet_fill, NULL);
