/* Talika panel applet: Switch between open windows using icons.
 * (C) 2009-2010 Sinew Software Systems
	 *
 * Authors:  Vinod Kumar Dahiya
 *           Hemant Kumar
 *
 * NOTE : A lot of code ( about 90% ) has been taken from libwnck tasklist.c
	 * we just renamed & modified it to suit our purpose.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
	 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <gtk/gtk.h>
#include <libwnck/libwnck.h>
#include <math.h>

#include "tasklist.h"
#include "task.h"
#include "xutils.h"
#include "tooltips.h"
#include "launcher.h"


static void 
talika_task_stop_glow (TalikaTask * task);


/*########################### Drag and drop ##################################*/

static gboolean 
talika_task_motion_timeout (gpointer data)
{
	WnckWorkspace *ws;
	TalikaTask *task = data;

	task->button_activate = 0;

	/* FIXME: THIS IS SICK AND WRONG AND BUGGY.  See the end of
	 * http://mail.gnome.org/archives/wm-spec-list/2005-July/msg00032.html
	 * There should only be *one* activate call.
	 */
	ws = wnck_window_get_workspace (task->window);
	if (ws
	    && ws !=
	    wnck_screen_get_active_workspace (wnck_screen_get_default ()))
	{
		wnck_workspace_activate (ws, task->dnd_timestamp);
	}
	wnck_window_activate_transient (task->window, task->dnd_timestamp);

	task->dnd_timestamp = 0;

	return FALSE;
}

static void 
talika_task_drag_leave (GtkWidget * widget,
                        GdkDragContext * context,
                        guint time, TalikaTask * task)
{
	if (task->button_activate != 0)
	{
		g_source_remove (task->button_activate);
		task->button_activate = 0;
	}

	gtk_drag_unhighlight (widget);
}

static gboolean 
talika_task_drag_motion (GtkWidget * widget,
                         GdkDragContext * context,
                         gint x, gint y, 
                         guint time, 
                         TalikaTask * task)
{
	if (gtk_drag_dest_find_target (widget, context, NULL))
	{
		gtk_drag_highlight (widget);
		gdk_drag_status (context, context->suggested_action, time);
	}
	else
	{
		task->dnd_timestamp = time;

		if (task->button_activate == 0
		    && task->type == TALIKA_TASK_WINDOW)
			task->button_activate =
			g_timeout_add (WNCK_ACTIVATE_TIMEOUT,
			               talika_task_motion_timeout,
			               task);
		gdk_drag_status (context, 0, time);
	}
	return TRUE;
}

static void 
talika_task_drag_begin (GtkWidget * widget,
                        GdkDragContext * context, TalikaTask * task)
{
	GdkPixbuf *buf = talika_task_get_icon (task);
	gtk_drag_set_icon_pixbuf (context, buf, -2, -2);
	task->tasklist->drag_start_time = gtk_get_current_event_time ();
}

static void
talika_task_drag_end (GtkWidget * widget,
                      GdkDragContext * context, TalikaTask * task)
{
	task->tasklist->drag_start_time = 0;
}

static void
talika_task_drag_data_get (GtkWidget * widget,
                           GdkDragContext * context,
                           GtkSelectionData * selection_data,
                           guint info, guint time, TalikaTask * task)
{
	gulong xid;

	if (task->type == TALIKA_TASK_WINDOW)
	{
		xid = wnck_window_get_xid (task->window);
	}
	else
	{
		TalikaTask *temp;
		temp = g_list_last (task->windows)->data;
		xid = wnck_window_get_xid (temp->window);
	}

	gtk_selection_data_set (selection_data,
	                        selection_data->target,
	                        8, (guchar *) & xid, sizeof (gulong));
}

static void
talika_task_drag_data_received (GtkWidget * widget,
                                GdkDragContext * context,
                                gint x,
                                gint y,
                                GtkSelectionData * data,
                                guint info,
                                guint time, TalikaTask * target_task)
{
	TalikaTaskList *tasklist;
	GList *l, *windows;
	WnckWindow *window;
	gulong *xid;
	guint new_order, old_order, order;
	WnckWindow *found_window;

	if (target_task->type == TALIKA_TASK_CLASS_GROUP)
	{
		target_task = g_list_last (target_task->windows)->data;
	}

	if ((data->length != sizeof (gulong)) || (data->format != 8))
	{
		gtk_drag_finish (context, FALSE, FALSE, time);
		return;
	}

	tasklist = target_task->tasklist;
	xid = (gulong *) data->data;
	found_window = NULL;
	new_order = 0;
	windows = wnck_screen_get_windows (tasklist->screen);

	for (l = windows; l; l = l->next)
	{
		window = WNCK_WINDOW (l->data);
		if (wnck_window_get_xid (window) == *xid)
		{
			old_order = wnck_window_get_sort_order (window);
			new_order =
				wnck_window_get_sort_order
				(target_task->window);
			if (old_order < new_order)
				new_order++;
			found_window = window;
			break;
		}
	}

	if (target_task->window == found_window)
	{
		GtkSettings *settings;
		gint double_click_time;

		settings =
			gtk_settings_get_for_screen (gtk_widget_get_screen
			                             (GTK_WIDGET (tasklist)));
		double_click_time = 0;
		g_object_get (G_OBJECT (settings),
		              "gtk-double-click-time", &double_click_time,
		              NULL);

		if ((time - tasklist->drag_start_time) < double_click_time)
		{
			talika_tasklist_activate_task_window (target_task,
			                                      time);
			gtk_drag_finish (context, TRUE, FALSE, time);
			return;
		}
	}

	if (found_window)
	{
		for (l = windows; l; l = l->next)
		{
			window = WNCK_WINDOW (l->data);
			order = wnck_window_get_sort_order (window);
			if (order >= new_order)
				wnck_window_set_sort_order (window,
				                            order + 1);
		}
		wnck_window_set_sort_order (found_window, new_order);

		if (!tasklist->include_all_workspaces)
		{
			WnckWorkspace *active_space;
			active_space =
				wnck_screen_get_active_workspace
				(tasklist->screen);
			wnck_window_move_to_workspace (found_window,
			                               active_space);
		}

		talika_tasklist_update_lists (tasklist);
		gtk_widget_queue_resize (GTK_WIDGET (tasklist->box));
	}

	gtk_drag_finish (context, TRUE, FALSE, time);
}



/*##################### Attention demanding task glowing #####################*/

static gboolean
talika_task_get_needs_attention (TalikaTask * task)
{
	GList *l;
	TalikaTask *win_task;
	gboolean needs_attention;

	needs_attention = FALSE;

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
		{
			task->start_needs_attention = 0;
			l = task->windows;
			while (l)
			{
				win_task = (TalikaTask *) l->data;

				if (wnck_window_or_transient_needs_attention
				    (win_task->window))
				{
					needs_attention = TRUE;
					time (&(task->start_needs_attention));
					break;
				}

				l = l->next;
			}
		}
			break;

		case TALIKA_TASK_WINDOW:
		{
			needs_attention =
				wnck_window_or_transient_needs_attention
				(task->window);
			if (needs_attention)
				time (&(task->start_needs_attention));
		}
			break;
	}

	return needs_attention != FALSE;
}

static gboolean
talika_task_button_glow (TalikaTask * task)
{
	GTimeVal tv;
	gdouble glow_factor, now;
	gfloat fade_opacity, loop_time;
	gint fade_max_loops;
	gboolean stopped;
	cairo_t *cr;

	if (task->screenshot == NULL)
		return TRUE;

	g_get_current_time (&tv);
	now = (tv.tv_sec * (1.0 * G_USEC_PER_SEC) +
	       tv.tv_usec) / G_USEC_PER_SEC;

	if (task->glow_start_time <= G_MINDOUBLE)
		task->glow_start_time = now;

	fade_opacity = 0.8;
	fade_max_loops = 5;
	loop_time = 3;

	if (task->button_glow == 0)
	{
		/* we're in "has stopped glowing" mode */
		glow_factor = 0;	//fade_opacity * 0.5;
		stopped = TRUE;
	}
	else
	{
		glow_factor = fade_opacity * (0.5 -
		                              0.5 *
		                              cos ((now -
		                                    task->glow_start_time) *
		                                   M_PI * 2.0 / loop_time));

		if (now - task->start_needs_attention >
		    loop_time * 1.0 * fade_max_loops)
			stopped =
			ABS (glow_factor - fade_opacity * 0.5) < 0.05;
		else
			stopped = FALSE;
	}

	gdk_window_begin_paint_rect (task->button->window,
	                             &task->button->allocation);

	cr = gdk_cairo_create (task->button->window);
	gdk_cairo_rectangle (cr, &task->button->allocation);
	cairo_translate (cr, task->button->allocation.x,
	                 task->button->allocation.y);
	cairo_clip (cr);

	cairo_save (cr);

	gdk_cairo_set_source_pixmap (cr, task->screenshot, 0., 0.);
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_paint (cr);

	cairo_restore (cr);

	gdk_cairo_set_source_pixmap (cr, task->screenshot_faded, 0., 0.);
	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
	cairo_paint_with_alpha (cr, glow_factor);

	cairo_destroy (cr);

	gdk_window_end_paint (task->button->window);

	if (stopped)
		talika_task_stop_glow (task);

	return !stopped;
}

static void
talika_task_clear_glow_start_timeout_id (TalikaTask * task)
{
	task->button_glow = 0;
}

static void
talika_task_queue_glow (TalikaTask * task)
{
	if (task->button_glow == 0)
	{
		task->glow_start_time = 0.0;

		/* The animation doesn't speed up or slow down based on the
		 * timeout value, but instead will just appear smoother or 
		 * choppier.
		 */
		task->button_glow =
			g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
			                    50,
			                    (GSourceFunc)
			                    talika_task_button_glow, task,
			                    (GDestroyNotify)
			                    talika_task_clear_glow_start_timeout_id);
	}
}

static void 
talika_task_stop_glow (TalikaTask * task)
{
	if (task->button_glow != 0)
		g_source_remove (task->button_glow);
}


static void
fake_expose_widget (GtkWidget * widget, GdkPixmap * pixmap, gint x, gint y)
{
	GdkWindow *tmp_window;
	GdkEventExpose event;

	event.type = GDK_EXPOSE;
	event.window = pixmap;
	event.send_event = FALSE;
	event.region = NULL;
	event.count = 0;

	tmp_window = widget->window;
	widget->window = pixmap;
	widget->allocation.x += x;
	widget->allocation.y += y;

	event.area = widget->allocation;

	gtk_widget_send_expose (widget, (GdkEvent *) & event);

	widget->window = tmp_window;
	widget->allocation.x -= x;
	widget->allocation.y -= y;
}

static GdkPixmap *
take_screenshot (TalikaTask * task)
{
	TalikaTaskList *tasklist;
	GtkWidget *tasklist_widget;
	GdkPixmap *pixmap;
	gint width, height;
	gboolean overlay_rect = TRUE;

	width = task->button->allocation.width;
	height = task->button->allocation.height;

	pixmap = gdk_pixmap_new (task->button->window, width, height, -1);

	tasklist = task->tasklist;
	tasklist_widget = GTK_WIDGET (task->tasklist->box);

	/* first draw the button */
	//gtk_widget_style_get (tasklist_widget, "fade-overlay-rect", &overlay_rect, NULL);
	if (overlay_rect)
	{
		/* Draw a rectangle with bg[SELECTED] */
		gdk_draw_rectangle (pixmap,
		                    task->button->style->
		                    bg_gc[GTK_STATE_SELECTED], TRUE, 0, 0,
		                    width + 1, height + 1);
	}
	else
	{
		GtkStyle *style;
		GtkStyle *attached_style;

		/* copy the style to change its colors around. */
		style = gtk_style_copy (task->button->style);
		style->bg[task->button->state] =
			style->bg[GTK_STATE_SELECTED];
		/* Now attach it to the window */
		attached_style = gtk_style_attach (style, pixmap);
		g_object_ref (attached_style);

		/* copy the background */
		gdk_draw_drawable (pixmap,
		                   attached_style->bg_gc[GTK_STATE_NORMAL],
		                   tasklist->background,
		                   task->button->allocation.x,
		                   task->button->allocation.y, 0, 0, width,
		                   height);

		/* draw the button with our modified style instead of the real one. */
		gtk_paint_box (attached_style, (GdkWindow *) pixmap,
		               task->button->state, GTK_SHADOW_OUT, NULL,
		               task->button, "button", 0, 0, width, height);

		g_object_unref (style);
		gtk_style_detach (attached_style);
		g_object_unref (attached_style);
	}

	/* then the image and label */
	fake_expose_widget (task->image, pixmap,
	                    -task->button->allocation.x,
	                    -task->button->allocation.y);
	//fake_expose_widget (task->label, pixmap,
	//                    -task->button->allocation.x, -task->button->allocation.y);

	return pixmap;
}

static GdkPixmap *
copy_pixmap (GtkWidget * widget)
{
	GdkPixmap *pixmap;

	pixmap = gdk_pixmap_new (widget->window,
	                         widget->allocation.width,
	                         widget->allocation.height, -1);

	gdk_draw_drawable (pixmap,
	                   widget->style->bg_gc[GTK_STATE_NORMAL],
	                   widget->window,
	                   widget->allocation.x, widget->allocation.y,
	                   0, 0,
	                   widget->allocation.width,
	                   widget->allocation.height);

	return pixmap;
}


static void
cleanup_screenshots (TalikaTask * task)
{
	if (task->screenshot != NULL)
	{
		g_object_unref (task->screenshot);
		task->screenshot = NULL;
	}
	if (task->screenshot_faded != NULL)
	{
		g_object_unref (task->screenshot_faded);
		task->screenshot_faded = NULL;
	}
}

static gboolean
talika_task_expose (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	GtkStyle *style;
	GdkGC *lgc, *dgc;
	int x, y, w;
	TalikaTask *task;

	task = (TalikaTask *) data;

	cleanup_screenshots (task);

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
		{
			style = widget->style;
			lgc = style->light_gc[GTK_STATE_NORMAL];
			dgc = style->dark_gc[GTK_STATE_NORMAL];

			x = widget->allocation.x + widget->allocation.width -
				(GTK_CONTAINER (widget)->border_width +
				 style->ythickness + 12);
			y = widget->allocation.y + widget->allocation.height / 2 - 5;

			//////////////////////////////// DRAW ARROW //////////////////////////////////
			GtkArrowType arrow_type = GTK_ARROW_UP;

			w = (widget->allocation.width -
			     GTK_CONTAINER (widget)->border_width -
			     style->ythickness * 2) / 4;

			switch (task->tasklist->panel_orient)
			{
				case PANEL_APPLET_ORIENT_UP:
				{
					x = widget->allocation.x + widget->allocation.width -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					y = widget->allocation.y +
						GTK_CONTAINER (widget)->border_width +
						style->ythickness;

					arrow_type = GTK_ARROW_UP;
				}
					break;

				case PANEL_APPLET_ORIENT_DOWN:
				{
					x = widget->allocation.x + widget->allocation.width -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					y = widget->allocation.y + widget->allocation.height -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					arrow_type = GTK_ARROW_DOWN;
				}
					break;

				case PANEL_APPLET_ORIENT_RIGHT:
				{
					x = widget->allocation.x + widget->allocation.width -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					y = widget->allocation.y + widget->allocation.height -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					arrow_type = GTK_ARROW_RIGHT;
				}
					break;

				case PANEL_APPLET_ORIENT_LEFT:
				{
					x = widget->allocation.x +
						GTK_CONTAINER (widget)->border_width +
						style->ythickness;

					y = widget->allocation.y + widget->allocation.height -
						(GTK_CONTAINER (widget)->border_width +
						 style->ythickness + w);

					arrow_type = GTK_ARROW_LEFT;
				}
					break;

				default:
					break;
			}

			gtk_paint_arrow (widget->style, widget->window,
			                 task->tasklist->active_class_group ==
			                 task ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL,
			                 GTK_SHADOW_OUT, NULL, widget, NULL,
			                 arrow_type, TRUE, x, y, w, w);
		}

			//////////////////////////////////////////////////////////////////////////////

			/* Fall through to get screenshot
			 */
		case TALIKA_TASK_WINDOW:
		{
			if ((event->area.x <= widget->allocation.x) &&
			    (event->area.y <= widget->allocation.y) &&
			    (event->area.width >= widget->allocation.width) &&
			    (event->area.height >= widget->allocation.height))
			{
				if (task->start_needs_attention)
				{
					task->screenshot = copy_pixmap (widget);
					task->screenshot_faded =
						take_screenshot (task);

					talika_task_button_glow (task);
				}
			}
		}
			break;
	}

	return FALSE;
}

/*################ Button press and Corrsponding Menu handling ###############*/

static void
talika_task_position_menu (GtkMenu * menu,
                           int *x,
                           int *y, gboolean * push_in, gpointer user_data)
{
	TalikaTask *task = (TalikaTask *) user_data;
	GtkWidget *widget = task->button;

	GtkRequisition requisition;
	GdkScreen *screen;
	int menu_x = 0;
	int menu_y = 0;
	int pointer_x;
	int pointer_y;

	screen = gtk_widget_get_screen (widget);

	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);

	gdk_window_get_origin (widget->window, &menu_x, &menu_y);
	gtk_widget_get_pointer (widget, &pointer_x, &pointer_y);

	menu_x += widget->allocation.x;
	menu_y += widget->allocation.y;

	if (task->tasklist->panel_orient == PANEL_APPLET_ORIENT_UP ||
	    task->tasklist->panel_orient == PANEL_APPLET_ORIENT_DOWN)
	{
		if (gtk_widget_get_direction (GTK_WIDGET (menu)) !=
		    GTK_TEXT_DIR_RTL)
		{
			if (pointer_x < widget->allocation.width &&
			    requisition.width < pointer_x)
				menu_x += MIN (pointer_x,
				               widget->allocation.width -
				               requisition.width);
		}
		else
		{
			menu_x +=
				widget->allocation.width - requisition.width;
			if (pointer_x > 0
			    && pointer_x < widget->allocation.width
			    && pointer_x <
			    widget->allocation.width - requisition.width)
			{
				menu_x -=
					MIN (widget->allocation.width -
					     pointer_x,
					     widget->allocation.width -
					     requisition.width);
			}
		}
		menu_x = MIN (menu_x,
		              gdk_screen_get_width (screen) -
		              requisition.width);

		if (menu_y > gdk_screen_get_height (screen) / 2)
			menu_y -= requisition.height;
		else
			menu_y += widget->allocation.height;
	}
	else
	{
		if (pointer_y < widget->allocation.height &&
		    requisition.height < pointer_y)
			menu_y +=
			MIN (pointer_y,
			     widget->allocation.height -
			     requisition.height);
		menu_y = MIN (menu_y,
		              gdk_screen_get_height (screen) -
		              requisition.height);

		if (menu_x > gdk_screen_get_width (screen) / 2)
			menu_x -= requisition.width;
		else
			menu_x += widget->allocation.width;

	}

	*x = menu_x;
	*y = menu_y;
	*push_in = TRUE;
}

static void
talika_task_close_all (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;
	GList *l;

	l = task->windows;
	while (l)
	{
		TalikaTask *child = l->data;
		wnck_window_close (child->window,
		                   gtk_get_current_event_time ());
		l = l->next;
	}
}

static void
talika_task_unminimize_all (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;
	GList *l;

	l = task->windows;
	while (l)
	{
		TalikaTask *child = l->data;

		//update cache first
		talika_preview_image_cache_update_window (child->tasklist->preview_cache,
		                                          child->window,
		                                          child->tasklist->preview_size,
		                                          child->tasklist->box);
		
		/* This is inside an activate callback, so gtk_get_current_event_time()
		* will work.
		 */
		wnck_window_unminimize (child->window,
		                        gtk_get_current_event_time ());
		l = l->next;
	}
}

static void
talika_task_minimize_all (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;
	GList *l;

	l = task->windows;
	while (l)
	{
		TalikaTask *child = l->data;

		//update cache first
		talika_preview_image_cache_update_window (child->tasklist->preview_cache,
		                                          child->window,
		                                          child->tasklist->preview_size,
		                                          child->tasklist->box);
		
		wnck_window_minimize (child->window);
		l = l->next;
	}
}

static void
talika_task_unmaximize_all (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;
	GList *l;

	l = task->windows;
	while (l)
	{
		TalikaTask *child = l->data;
		wnck_window_unmaximize (child->window);
		l = l->next;
	}
}

static void
talika_task_maximize_all (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;
	GList *l;

	l = task->windows;
	while (l)
	{
		TalikaTask *child = l->data;
		wnck_window_maximize (child->window);
		l = l->next;
	}
}

static void
talika_task_menu_activated (GtkMenuItem * menu_item, gpointer data)
{
	TalikaTask *task = data;

	/* This is an "activate" callback function so gtk_get_current_event_time()
	* will suffice.
	 */
	talika_tasklist_activate_task_window (task,
	                                      gtk_get_current_event_time ());
}

static void 
talika_task_pin (GtkMenuItem *menu_item, TalikaTask *task)
{
	talika_tasklist_add_launcher (task);
}

static void
talika_task_add_launcher_menuitems (GtkWidget *menu, TalikaTask *task)
{
	GtkWidget *menu_item;

	menu_item = gtk_separator_menu_item_new();
	gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menu_item);
	
	if(task->launcher)
	{
		menu_item = gtk_menu_item_new_with_mnemonic (_("Launch"));
		gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item),
		                  "activate",
		                  G_CALLBACK (talika_launcher_launch), task->launcher);
		
		menu_item = gtk_menu_item_new_with_mnemonic (_("Unpin"));
		gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item),
		                  "activate",
		                  G_CALLBACK (talika_launcher_unpin), task->launcher);
	}
	else
	{
		menu_item = gtk_menu_item_new_with_mnemonic (_("Pin"));
		gtk_menu_shell_prepend (GTK_MENU_SHELL(menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item),
		                  "activate",
		                  G_CALLBACK (talika_task_pin), task);
	}
}

static void
talika_task_popup_menu (TalikaTask * task, gboolean action_submenu)
{
	GtkWidget *menu;
	TalikaTask *win_task;
	char *text;
	GdkPixbuf *pixbuf;
	GtkWidget *menu_item;
	GtkWidget *image;
	GList *l, *list;

	g_return_if_fail (task->type == TALIKA_TASK_CLASS_GROUP);

	if (task->class_group == NULL)
		return;

	if (task->menu == NULL)
	{
		task->menu = gtk_menu_new ();
		g_object_ref_sink (task->menu);
	}

	menu = task->menu;

	/* Remove old menu content */
	list = gtk_container_get_children (GTK_CONTAINER (menu));
	l = list;
	while (l)
	{
		GtkWidget *child = GTK_WIDGET (l->data);
		gtk_container_remove (GTK_CONTAINER (menu), child);
		l = l->next;
	}
	g_list_free (list);

	l = task->windows;
	while (l)
	{
		win_task = l->data;

		text = talika_task_get_text (win_task);
		menu_item = gtk_image_menu_item_new_with_label (text);
		g_free (text);

		//gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM
		//                                         (menu_item), TRUE);

		if (talika_task_get_needs_attention (win_task))
			_make_gtk_label_bold (GTK_LABEL
			                      (GTK_BIN (menu_item)->child));

		text = talika_task_get_text (win_task);
		gtk_widget_set_tooltip_text (menu_item, text);
		g_free (text);

		pixbuf = gdk_pixbuf_copy (wnck_window_get_mini_icon
		                          (win_task->window));
		if (pixbuf)
		{
			image = gtk_image_new_from_pixbuf (pixbuf);
			gtk_widget_show (image);
			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
			                               (menu_item), image);
			g_object_unref (pixbuf);
		}

		gtk_widget_show (menu_item);

		if (action_submenu)
			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
			                           wnck_action_menu_new
			                           (win_task->window));
		else
		{
			g_signal_connect (G_OBJECT (menu_item), "activate",
			                  G_CALLBACK
			                  (talika_task_menu_activated),
			                  win_task);
		}

		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

		l = l->next;
	}

	/* In case of Right click, show Minimize All, Unminimize All, Close All */
	if (action_submenu)
	{
		GtkWidget *separator;
		GtkWidget *image;

		separator = gtk_separator_menu_item_new ();
		gtk_widget_show (separator);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator);

		menu_item =
			gtk_image_menu_item_new_with_mnemonic (_
			                                       ("Mi_nimize All"));
		image = gtk_image_new_from_stock (WNCK_STOCK_MINIMIZE,
		                                  GTK_ICON_SIZE_MENU);
		gtk_widget_show (image);
		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
		                               (menu_item), image);
		gtk_widget_show (menu_item);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
		                  G_CALLBACK (talika_task_minimize_all),
		                  task);

		menu_item =
			gtk_image_menu_item_new_with_mnemonic (_
			                                       ("Un_minimize All"));
		gtk_widget_show (menu_item);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
		                  G_CALLBACK (talika_task_unminimize_all),
		                  task);

		menu_item =
			gtk_image_menu_item_new_with_mnemonic (_
			                                       ("Ma_ximize All"));
		image = gtk_image_new_from_stock (WNCK_STOCK_MAXIMIZE,
		                                  GTK_ICON_SIZE_MENU);
		gtk_widget_show (image);
		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
		                               (menu_item), image);
		gtk_widget_show (menu_item);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
		                  G_CALLBACK (talika_task_maximize_all),
		                  task);

		menu_item =
			gtk_image_menu_item_new_with_mnemonic (_
			                                       ("_Unmaximize All"));
		gtk_widget_show (menu_item);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
		                  G_CALLBACK (talika_task_unmaximize_all),
		                  task);

		separator = gtk_separator_menu_item_new ();
		gtk_widget_show (separator);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator);

		menu_item =
			gtk_image_menu_item_new_with_mnemonic (_
			                                       ("_Close All"));
		image = gtk_image_new_from_stock (WNCK_STOCK_DELETE,
		                                  GTK_ICON_SIZE_MENU);
		gtk_widget_show (image);
		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
		                               (menu_item), image);
		gtk_widget_show (menu_item);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
		                  G_CALLBACK (talika_task_close_all), task);

		talika_task_add_launcher_menuitems (menu, task);
	}

	gtk_menu_set_screen (GTK_MENU (menu),
	                     talika_screen_get_gdk_screen (task->
	                                                   tasklist->screen));

	gtk_widget_show_all (menu);
	gtk_menu_popup (GTK_MENU (menu),
	                NULL, NULL,
	                talika_task_position_menu, task,
	                1, gtk_get_current_event_time ());
}

static gboolean
talika_task_button_press_event (GtkWidget * widget,
                                GdkEventButton * event, gpointer data)
{
	TalikaTask *task = data;

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
		{
			if (event->button == 1)
			{
				talika_tooltips_hide_popup_immediatly (task);
				return FALSE;
			}
			else if (event->button == 2)
			{

			}
			else if (event->button == 3)
			{
				talika_tooltips_hide_popup_immediatly (task);
				talika_task_popup_menu (task, TRUE);
			}
			return TRUE;
		}

		case TALIKA_TASK_WINDOW:
		{
			if (event->button == 1)
			{

				/* is_most_recently_activated == is_active for click &
				 * sloppy focus methods.  We use the former here because
				 * 'mouse' focus provides a special case.  In that case, no
				 * window will be active, but if a window was the most
				 * recently active one (i.e. user moves mouse straight from
				                        * window to tasklist), then we should still minimize it.
											*/
				if (wnck_window_is_most_recently_activated
				    (task->window))
					task->was_active = TRUE;
				else
					task->was_active = FALSE;

				return FALSE;
			}
			else if (event->button == 2)
			{
				talika_tooltips_hide_popup_immediatly (task);
				wnck_window_close (task->window,
				                   gtk_get_current_event_time ());
			}
			else if (event->button == 3)
			{
				talika_tooltips_hide_popup_immediatly (task);

				if (task->action_menu)
					gtk_widget_destroy (task->action_menu);

				g_assert (task->action_menu == NULL);

				task->action_menu =
					wnck_action_menu_new (task->window);

				talika_task_add_launcher_menuitems (task->action_menu, task);

				g_object_add_weak_pointer (G_OBJECT
				                           (task->action_menu),
				                           (void **)
				                           &task->action_menu);

				gtk_menu_set_screen (GTK_MENU (task->action_menu),
				                     talika_screen_get_gdk_screen
				                     (task->tasklist->screen));

				gtk_widget_show_all (task->action_menu);
				gtk_menu_popup (GTK_MENU (task->action_menu),
				                NULL, NULL,
				                talika_task_position_menu,
				                task, event->button,
				                gtk_get_current_event_time ());

				g_signal_connect (task->action_menu, "selection-done",
				                  G_CALLBACK (gtk_widget_destroy),
				                  NULL);

				return TRUE;
			}
		}
	}
	return FALSE;
}

static void
talika_task_button_toggled (GtkButton * button, TalikaTask * task)
{
	/* Did we really want to change the state of the togglebutton? */
	if (task->really_toggling)
		return;

	/* Undo the toggle */
	task->really_toggling = TRUE;
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
	                              !gtk_toggle_button_get_active
	                              (GTK_TOGGLE_BUTTON (button)));
	task->really_toggling = FALSE;

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
			talika_task_popup_menu (task, FALSE);
			break;

		case TALIKA_TASK_WINDOW:
			if (task->window == NULL)
				return;

			/* This should only be called by clicking on the task button, so
			 * gtk_get_current_event_time() should be fine here...
				 */
			talika_tasklist_activate_task_window (task,
			                                      gtk_get_current_event_time());
			break;
	}
}


/*########################## Manage Button Icons #############################*/
GdkPixbuf*
talika_desaturate_icon (GdkPixbuf *pixbuf)
{
	GdkPixbuf *desat = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
                            TRUE,
                            gdk_pixbuf_get_bits_per_sample (pixbuf),
                            gdk_pixbuf_get_width (pixbuf),
                            gdk_pixbuf_get_height (pixbuf));

	gdk_pixbuf_saturate_and_pixelate (pixbuf,
                                        desat,
                                        0,
                                       FALSE);

	return desat;
}

static void
talika_dimm_icon (GdkPixbuf ** pixbufPtr)
{
	int x, y, pixel_stride, row_stride;
	guchar *row, *pixels;
	int w, h;

	GdkPixbuf *pixbuf = *pixbufPtr;

	//talika exits on startup sometime because icon is null
	//checking here will handle all cases
	if(pixbuf == NULL)
		return;

	w = gdk_pixbuf_get_width (pixbuf);
	h = gdk_pixbuf_get_height (pixbuf);

	if (!gdk_pixbuf_get_has_alpha (pixbuf))
	{
		GdkPixbuf *temp =
			gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
		g_object_unref (pixbuf);
		*pixbufPtr = temp;
		pixbuf = temp;
	}

	pixel_stride = 4;

	row = gdk_pixbuf_get_pixels (pixbuf);
	row_stride = gdk_pixbuf_get_rowstride (pixbuf);

	for (y = 0; y < h; y++)
	{
		pixels = row;

		for (x = 0; x < w; x++)
		{
			pixels[3] /= 2;

			pixels += pixel_stride;
		}

		row += row_stride;
	}
}

/*static GdkPixbuf *
talika_get_installed_app_icon (TalikaTask * task, guint size)
{
	GError *err = NULL;
	GAppInfo *app = g_app_info_create_from_commandline ("gimp",NULL,
	                                                    G_APP_INFO_CREATE_NONE,
	                                                    &err);

	if(err)
	{
		g_print("error\n");
	}

	g_print("%s",g_app_info_get_description(app));
	g_print("\n");
	GIcon *icon = g_app_info_get_icon (app);
	if(!icon)
	{
		g_print("no icon\n");
	}
	//g_print( g_icon_to_string (icon));
	
	return NULL;
}
*/

static GdkPixbuf *
talika_get_hq_icon (TalikaTask * task, guint size)
{
	if (task->type == TALIKA_TASK_WINDOW)
		return talika_get_icon_from_xserver (task->window, size);

	return NULL;
}

static GdkPixbuf *
talika_sized_pixbuf_for_task (TalikaTask * task, gint size)
{
	GdkPixbuf *pbuf = NULL;

	if (task->type == TALIKA_TASK_WINDOW)
		pbuf = gdk_pixbuf_copy (wnck_window_get_icon (task->window));
	else
		pbuf = gdk_pixbuf_copy (wnck_class_group_get_icon
		                        (task->class_group));

	gint width = gdk_pixbuf_get_width (pbuf);
	gint height = gdk_pixbuf_get_height (pbuf);

	if (MAX (width, height) != size)
	{
		gdouble scale =
			(gdouble) size / (gdouble) MAX (width, height);

		GdkPixbuf *tmp = pbuf;
		pbuf = gdk_pixbuf_scale_simple (tmp, (gint) (width * scale),
		                                (gint) (height * scale),
		                                GDK_INTERP_HYPER);

		g_object_unref (tmp);
	}

	return pbuf;
}

static GdkPixbuf *
talika_task_get_window_icon (TalikaTask * task)
{
	GdkPixbuf *pixbuf = NULL;

	guint size = task->tasklist->panel_size;
	size -= 8;		//subtract box and button paddings

	//first look if theme has icon, dimm icon may fail 
	//pixbuf = talika_get_installed_app_icon (task, size);

	if (!pixbuf)
	{
		//second ask x11 for icon
		pixbuf = talika_get_hq_icon (task, size);

		//finaly do the trick
		if (!pixbuf)
		{
			//g_print ("from wnck : %s\n",
			  //       wnck_window_get_name (task->window));

			if (size >= 16 && size < 19)
			{
				pixbuf = gdk_pixbuf_copy
					(wnck_window_get_mini_icon
					 (task->window));
			}
			else if (size > 30 && size < 36)
			{
				pixbuf = gdk_pixbuf_copy (wnck_window_get_icon
				                          (task->window));
			}
			else
				pixbuf = talika_sized_pixbuf_for_task
				(task, size);
		}
		else
		{
			//g_print ("from xwin : %s\n",
			  //       wnck_window_get_name (task->window));
		}
	}
	else
	{
		//g_print ("from theme : %s\n",
		 //        wnck_window_get_name (task->window));
	}

	if (task->tasklist->dimm_minimised &&
	    wnck_window_is_minimized (task->window))
		talika_dimm_icon (&pixbuf);
	
	return pixbuf;
}

static GdkPixbuf *
talika_task_get_class_group_icon (TalikaTask * task)
{
	GdkPixbuf *pixbuf = NULL;

	guint size = task->tasklist->panel_size;
	size -= 8;		//subtract box and button paddings

	//first look if a desktop file exists
	//DISABLED FOR NOW AS IT TAKES LONGER TO REFRESH
	/*gint pid = 0;

	if(task->windows)
	{
		WnckWindow *first_win = ((TalikaTask *)(task->windows->data))->window;
		pid = wnck_window_get_pid (first_win);
	}
	
	pixbuf = talika_get_icon_from_desktop_file (wnck_class_group_get_res_class 
	                                     (task->class_group),
	                                       pid, size);
*/
	//finaly do the trick
	if (!pixbuf)
	{
		//g_print ("from wnck : %s\n",
		//         wnck_class_group_get_name (task->class_group));

		if (size >= 16 && size < 19)
		{
			pixbuf = gdk_pixbuf_copy
				(wnck_class_group_get_mini_icon
				 (task->class_group));
		}
		else if (size > 30 && size < 36)
		{
			pixbuf = gdk_pixbuf_copy (wnck_class_group_get_icon
			                          (task->class_group));
		}
		else
			pixbuf = talika_sized_pixbuf_for_task (task, size);
	}
	else
	{
		//g_print ("from theme : %s\n",
		//         wnck_class_group_get_name (task->class_group));
	}

	GList *l;
	gboolean all_minimised = TRUE;
	l = task->windows;
	while (l != NULL)
	{
		TalikaTask *task1 = (TalikaTask *) l->data;
		l = l->next;

		if (!wnck_window_is_minimized (task1->window))
		{
			all_minimised = FALSE;
			break;
		}
	}

	if (task->tasklist->dimm_minimised && all_minimised)
		talika_dimm_icon (&pixbuf);


	return pixbuf;
}

GdkPixbuf*
talika_task_get_icon (TalikaTask * task)
{
	GdkPixbuf *pixbuf = NULL;

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
			pixbuf = talika_task_get_class_group_icon (task);
			break;

		case TALIKA_TASK_WINDOW:
			pixbuf = talika_task_get_window_icon (task);
			break;
	}

	if(task->tasklist->mono_icons && pixbuf)
	{
		GdkPixbuf *temp = pixbuf;
		pixbuf = talika_desaturate_icon (temp);
		g_object_unref (temp);
	}
	
	return pixbuf;
}

gchar*
talika_task_get_text (TalikaTask * task)
{
	const char *name;

	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
			name = wnck_class_group_get_name (task->class_group);
			if (name[0] != 0)
				return g_strdup_printf ("%s (%d)",
				                        name,
				                        g_list_length
				                        (task->windows));
			else
				return g_strdup_printf ("(%d)",
				                        g_list_length
				                        (task->windows));

		case TALIKA_TASK_WINDOW:
			name = wnck_window_get_name (task->window);

			if (name)
				return g_strdup (name);

			break;
	}

	return NULL;
}

/*###################### Task proprty change Callbacks #######################*/

void 
talika_task_update_visible_state (TalikaTask * task)
{
	GdkPixbuf *pixbuf;

	gtk_button_set_relief (GTK_BUTTON (task->button),
	                       task->
	                       tasklist->button_relief ? GTK_RELIEF_NORMAL :
		                       GTK_RELIEF_NONE);

	pixbuf = talika_task_get_icon (task);
	gtk_image_set_from_pixbuf (GTK_IMAGE (task->image), pixbuf);
	if (pixbuf)
		g_object_unref (pixbuf);

	if (talika_task_get_needs_attention (task))
	{
		talika_task_queue_glow (task);
	}
	else
	{
		talika_task_stop_glow (task);
	}

	gtk_widget_queue_resize (GTK_WIDGET (task->tasklist->box));
}

void 
do_talika_task_update_visible_state (gpointer data, 
                                     gpointer user_data)
{
	TalikaTask *task = (TalikaTask *) data;
	talika_task_update_visible_state (task);
}

static gboolean
talika_task_update_preview_cache_timeout (gpointer data)
{
	TalikaTask *task = (TalikaTask *) data;

	task->preview_update_timeout_id = 0;
	
	//update cache
	talika_preview_image_cache_update_window (task->tasklist->preview_cache,
	                                          task->window,
	                                          task->tasklist->preview_size,
	                                          task->tasklist->box);

	return FALSE;
}

void 
talika_task_state_changed (WnckWindow * window,
                           WnckWindowState changed_mask,
                           WnckWindowState new_state, gpointer data)
{
	TalikaTaskList *tasklist = data;

	//state changed, update cache
	if (!(new_state & WNCK_WINDOW_STATE_MINIMIZED) &&
	    !(new_state & WNCK_WINDOW_STATE_SKIP_TASKLIST))
	{
		TalikaTask *task = NULL;
		WnckWindow *win_temp = window;
		// FIXME: Handle group modal dialogs 
		for (; win_temp && !task;
		     win_temp = wnck_window_get_transient (win_temp))
			task =
			g_hash_table_lookup (tasklist->win_hash,
			                     win_temp);

		if(task)
		{
			if (task->preview_update_timeout_id)
				g_source_remove (task->preview_update_timeout_id);

			task->preview_update_timeout_id =
				g_timeout_add (500, &talika_task_update_preview_cache_timeout,
				               task);
		}
	}
	///////////////////////////////

	if (changed_mask & WNCK_WINDOW_STATE_SKIP_TASKLIST)
	{
		talika_tasklist_update_lists (tasklist);
		gtk_widget_queue_resize (GTK_WIDGET (tasklist->box));
		return;
	}

	if ((changed_mask & WNCK_WINDOW_STATE_DEMANDS_ATTENTION) ||
	    (changed_mask & WNCK_WINDOW_STATE_URGENT))
	{
		WnckWorkspace *active_workspace =
			wnck_screen_get_active_workspace (tasklist->screen);

		if (active_workspace &&
		    (active_workspace !=
		     wnck_window_get_workspace (window)
		     || (wnck_workspace_is_virtual (active_workspace)
		         && !wnck_window_is_in_viewport (window,
		                                         active_workspace))))
		{
			talika_tasklist_update_lists (tasklist);
			gtk_widget_queue_resize (GTK_WIDGET (tasklist->box));
		}
	}

	if ((changed_mask & WNCK_WINDOW_STATE_MINIMIZED) ||
	    (changed_mask & WNCK_WINDOW_STATE_DEMANDS_ATTENTION) ||
	    (changed_mask & WNCK_WINDOW_STATE_URGENT))
	{
		TalikaTask *win_task = NULL;

		// FIXME: Handle group modal dialogs 
		for (; window && !win_task;
		     window = wnck_window_get_transient (window))
			win_task =
			g_hash_table_lookup (tasklist->win_hash,
			                     window);

		if (win_task)
		{
			talika_task_update_visible_state (win_task);

			TalikaTask *class_group_task;

			class_group_task =
				g_hash_table_lookup
				(tasklist->class_group_hash,
				 win_task->class_group);

			if (class_group_task)
				talika_task_update_visible_state
				(class_group_task);
		}
	}
}

static void
talika_task_icon_changed (WnckWindow * window, gpointer data)
{
	TalikaTask *task = data;

	if (task)
		talika_task_update_visible_state (task);
}

static void
talika_task_name_changed (WnckWindow * window, gpointer data)
{
	TalikaTask *task = data;

	if (task)
		talika_task_update_visible_state (task);
}

static void
talika_task_class_name_changed (WnckClassGroup * class_group, gpointer data)
{
	TalikaTask *task = data;

	if (task)
		talika_task_update_visible_state (task);
}

static void
talika_task_class_icon_changed (WnckClassGroup * class_group, gpointer data)
{
	TalikaTask *task = data;

	if (task)
		talika_task_update_visible_state (task);
}

/*################################ Task ######################################*/

static void
talika_task_create_widgets (TalikaTask * task)
{
	GdkPixbuf *pixbuf;

	static const GtkTargetEntry targets[] = {
		{"application/x-wnck-window-id", 0, 0}
	};

	task->button = gtk_toggle_button_new ();

	g_object_add_weak_pointer (G_OBJECT (task->button),
	                           (void **) &task->button);

	gtk_widget_set_name (task->button, "tasklist-button");


	gtk_drag_source_set (GTK_WIDGET (task->button),
	                     GDK_BUTTON1_MASK, targets, 1, GDK_ACTION_MOVE);
	gtk_drag_dest_set (GTK_WIDGET (task->button),
	                   GTK_DEST_DEFAULT_DROP, targets, 1,
	                   GDK_ACTION_MOVE);

	gtk_button_set_relief (GTK_BUTTON (task->button),
	                       task->tasklist->button_relief ?
	                       GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);

	/*//some gtk styles have big button thickness, it tends to
	//distort either button or image
	//change button style here
	{
		GtkRcStyle *mod;

		mod = gtk_widget_get_modifier_style (task->button);
		mod->xthickness = 1;
		mod->ythickness = 1;

		gtk_widget_modify_style (task->button, mod);
	}*/

	pixbuf = talika_task_get_icon (task);
	if (pixbuf)
	{
		task->image = gtk_image_new_from_pixbuf (pixbuf);
		g_object_unref (pixbuf);
	}
	else
		task->image = gtk_image_new ();

	gtk_widget_show (task->image);

	gtk_container_add (GTK_CONTAINER (task->button), task->image);
	gtk_widget_show_all (task->button);


	if (talika_task_get_needs_attention (task))
	{
		talika_task_queue_glow (task);
	}

	g_signal_connect (G_OBJECT (task->button),
	                  "button_press_event",
	                  G_CALLBACK (talika_task_button_press_event), task);

	g_signal_connect (G_OBJECT (task->button), "toggled",
	                  G_CALLBACK (talika_task_button_toggled), task);

	int i = 0;
	switch (task->type)
	{
		case TALIKA_TASK_CLASS_GROUP:
			task->class_connections[i++] =
				g_signal_connect (G_OBJECT (task->class_group),
				                  "name_changed",
				                  G_CALLBACK
				                  (talika_task_class_name_changed),
				                  task);
			task->class_connections[i++] =
				g_signal_connect (G_OBJECT (task->class_group),
				                  "icon_changed",
				                  G_CALLBACK
				                  (talika_task_class_icon_changed),
				                  task);
			break;

		case TALIKA_TASK_WINDOW:
			i = 0;
			task->window_connections[i++] =
				g_signal_connect (G_OBJECT (task->window),
				                  "state_changed",
				                  G_CALLBACK
				                  (talika_task_state_changed),
				                  task->tasklist);
			task->window_connections[i++] =
				g_signal_connect (G_OBJECT (task->window),
				                  "icon_changed",
				                  G_CALLBACK
				                  (talika_task_icon_changed), task);
			task->window_connections[i++] =
				g_signal_connect (G_OBJECT (task->window),
				                  "name_changed",
				                  G_CALLBACK
				                  (talika_task_name_changed), task);
			break;
	}

	g_signal_connect (G_OBJECT (task->button), "drag_motion",
	                  G_CALLBACK (talika_task_drag_motion), task);


	g_signal_connect (G_OBJECT (task->button),
	                  "drag_data_get",
	                  G_CALLBACK (talika_task_drag_data_get), task);

	g_signal_connect (G_OBJECT (task->button),
	                  "drag_data_received",
	                  G_CALLBACK (talika_task_drag_data_received), task);

	g_signal_connect (G_OBJECT (task->button),
	                  "drag_begin",
	                  G_CALLBACK (talika_task_drag_begin), task);

	g_signal_connect (G_OBJECT (task->button), "drag_end",
	                  G_CALLBACK (talika_task_drag_end), task);


	g_signal_connect (G_OBJECT (task->button), "drag_leave",
	                  G_CALLBACK (talika_task_drag_leave), task);


	g_signal_connect_after (task->button, "expose_event",
	                        G_CALLBACK (talika_task_expose), task);

	g_signal_connect_after (task->button, "enter-notify-event",
	                        G_CALLBACK (talika_tooltips_enter_notify), task);

	g_signal_connect_after (task->button, "leave-notify-event",
	                        G_CALLBACK (talika_tooltips_leave_notify),
	                        task->tasklist);

}

TalikaTask* 
talika_task_new_from_window (TalikaTaskList * tasklist,
                             WnckWindow * window)
{
	TalikaTask *task;

	task = g_new0 (TalikaTask, 1);

	task->type = TALIKA_TASK_WINDOW;
	task->window = window;
	task->class_group = wnck_window_get_class_group (window);
	task->tasklist = tasklist;
	task->launcher = NULL;

	talika_task_create_widgets (task);
	return task;
}

TalikaTask* 
talika_task_new_from_class_group (TalikaTaskList * tasklist,
                                  WnckClassGroup * class_group)
{
	TalikaTask *task;

	task = g_new0 (TalikaTask, 1);

	task->type = TALIKA_TASK_CLASS_GROUP;
	task->window = NULL;
	task->class_group = class_group;
	task->tasklist = tasklist;
	task->launcher = NULL;

	talika_task_create_widgets (task);
	return task;
}

void 
talika_task_free (TalikaTask * task)
{
	talika_tooltips_hide_popup_immediatly (task);
	
	if (task->tasklist->active_task == task)
		talika_tasklist_change_active_task (task->tasklist, NULL);

	if (task->button)
	{
		g_object_remove_weak_pointer (G_OBJECT (task->button),
		                              (void **) &task->button);
		gtk_widget_destroy (task->button);
		task->button = NULL;
		task->image = NULL;
	}

	//disconnect signals
	int i = 0;
	while (i < N_WINDOW_CONNECTIONS)
	{
		if (task->window_connections[i] != 0)
			g_signal_handler_disconnect (G_OBJECT
			                             (task->window),
			                             task->window_connections
			                             [i]);
		task->window_connections[i] = 0;
		++i;
	}

	if (task->menu)
	{
		gtk_widget_destroy (task->menu);
		task->menu = NULL;
	}

	if (task->action_menu)
	{
		g_object_remove_weak_pointer (G_OBJECT
		                              (task->action_menu),
		                              (void **) &task->action_menu);
		gtk_widget_destroy (task->action_menu);
		task->action_menu = NULL;
	}

	task->window = NULL;

	g_list_free (task->windows);
	task->windows = NULL;

	//disconnect class connections
	i = 0;
	while (i < N_CLASS_CONNECTIONS)
	{
		if (task->class_connections[i] != 0)
			g_signal_handler_disconnect (G_OBJECT
			                             (task->class_group),
			                             task->class_connections
			                             [i]);
		task->class_connections[i] = 0;
		++i;
	}

	talika_task_stop_glow (task);

	cleanup_screenshots (task);

	if (task->preview_update_timeout_id != 0)
	{
		g_source_remove (task->preview_update_timeout_id);
		task->preview_update_timeout_id = 0;
	}

	g_free (task);
	task=NULL;
}
